"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Debug = require("debug");
const genkey_service_1 = require("../services/genkey.service");
const cache_service_1 = require("../services/cache.service");
const pool_service_1 = require("../services/pool.service");
const Config_Context_1 = require("../services/Config.Context");
const request_service_1 = require("../services/request.service");
const utils_1 = require("../utils");
const proxy_1 = require("../proxy");
const pathToRegExp = require('path-to-regexp');
function createNameSpaceHandler(namespace, options) {
    const logger = Debug(`acp:handler(${namespace})`);
    logger(`Register ${namespace}`);
    const namespacePath = pathToRegExp(options.expose, [], {
        sensitive: false,
        strict: false,
        end: false
    });
    const configContext = new Config_Context_1.ConfigContext(options.cache, options.rules);
    const proxyRequest = proxy_1.createProxy(options);
    return async function handler(ctx, next) {
        if (!namespacePath.test(ctx.path))
            return next();
        logger(`${ctx.path} matched!!`);
        const pathToCall = ctx.path.match(namespacePath)[1];
        logger(`${pathToCall} will be processed!`);
        const cacheConfig = configContext.getCacheConfig(pathToCall);
        const proxyPath = pathToCall + (ctx.search || '');
        if ((ctx.method !== 'GET') || !cacheConfig) {
            logger(`${ctx.method} ${pathToCall} ${cacheConfig ? '(No cache config)' : ''}`);
            const proxyCtxN = await proxyRequest(proxyPath, ctx.method, ctx.headers);
            return proxyCtxN.pipes(ctx);
        }
        const cacheKey = genkey_service_1.generateKey(namespace, ctx, cacheConfig);
        logger(`Cache Check ${pathToCall}`);
        try {
            return await cache_service_1.Cache.get(cacheKey).then(utils_1.respondWithCtx(ctx));
        }
        catch (e) { }
        logger(`Cache miss ${pathToCall}`);
        if (!cacheConfig.pool) {
            logger(`No request pool`);
            const proxyCtxM = await proxyRequest(proxyPath, ctx.method, ctx.headers);
            proxyCtxM.toPromise(ctx).then((res) => cache_service_1.Cache.put(cacheKey, res.toJSON(), +cacheConfig.expires))
                .catch(() => ({}));
            return proxyCtxM.pipes(ctx);
        }
        if (pool_service_1.RequestPool.has(cacheKey)) {
            return pool_service_1.RequestPool.wait(cacheKey).then(utils_1.respondWithCtx(ctx));
        }
        pool_service_1.RequestPool.add(cacheKey);
        const proxyCtx = await proxyRequest(proxyPath, ctx.method, ctx.headers);
        proxyCtx.toPromise(ctx)
            .then((res) => pool_service_1.RequestPool.putAndPublish(cacheKey, res.toJSON(), +cacheConfig.expires), (error) => {
            const response = request_service_1.errorToData(error);
            pool_service_1.RequestPool.errorAndPublish(cacheKey, response);
        })
            .catch((error) => {
            const response = request_service_1.errorToData(error);
            pool_service_1.RequestPool.errorAndPublish(cacheKey, response);
        });
        return proxyCtx.pipes(ctx);
    };
}
exports.createNameSpaceHandler = createNameSpaceHandler;
